;; The first three lines of this file were inserted by DrRacket. They record metadata
;; about the language level of this file in a form that our tools can easily process.
#reader(lib "htdp-beginner-reader.ss" "lang")((modname fsm-tues) (read-case-sensitive #t) (teachpacks ()) (htdp-settings #(#t write repeating-decimal #f #t none #f () #f)))
;; finite state machine example
(require "extras.rkt")
(require rackunit)


;; ps01 example was (a | b)* c (a | b)* d (e | f)*

;; The legal inputs to the machine are precisely the strings "a", "b",
;; "c", "d", "e", and "f". Any other inputs violate the machine's
;; contract, and its behavior on those inputs is unspecified.

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;; A MachineInput is one of
;; -- "a", "b", "c", "d", "e", and "f"
;; Interpretation: None- these don't refer to anything

;; TEMPLATE
;; input-fn : MachineInput -> ??
;; (define (input-fn inp)
;;   (cond
;;     [(string=? inp "a") ...]
;;     [(string=? inp "b") ...]
;;     [(string=? inp "c") ...]
;;     [(string=? inp "d") ...]
;;     [(string=? inp "e") ...]
;;     [(string=? inp "f") ...]))

;; with this template, an illegal input causes a Racket error.


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; ps01 example was (a | b)* c (a | b)* d ( e | f)*

;; A State is one of
;; "state0"     interp: what we've seen so far matches (a | b)*
;; "state1"     interp: what we've seen so far matches (a | b)*c(a|b)*
;; "state2"     interp: what we've seen so far matches (a | b)*c(a|b)*d(e|f)*
;; "errorstate" interp: what we have seen is something other than 
;;                      (a | b)*, (a | b)*c(a|b)*, or (a | b)*c(a|b)*d(e|f)*

;; Template:

;; state-fn : State -> ??
;; (define (state-fn s)
;;   (cond
;;     [(string=? s "state0") ...]
;;     [(string=? s "state1") ...]
;;     [(string=? s "state2") ...]
;;     [(string=? s "errorstate") ...]))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; next-state : State Input -> State
;; RETURNS: the state that should follow the given state after the
;; given input.
;; EXAMPLES: see interpretation of State
;; STRATEGY: Use template for State on s
(define (next-state s inp)
  (cond
    [(string=? s "state0") (after-state0 inp)]
    [(string=? s "state1") (after-state1 inp)]
    [(string=? s "state2") (after-state2 inp)]
    [(string=? s "errorstate") "errorstate"]))

;; after-state0: MachineInput -> State
;; GIVEN: a MachineInput
;; RETURNS: the state that should follow state0 after the given input
;; EXAMPLES: see next-state
;; STRATEGY: Use template for MachineInput on inp
(define (after-state0 inp)
  (cond
    [(string=? inp "a") "state0"]
    [(string=? inp "b") "state0"]
    [(string=? inp "c") "state1"]
    [(string=? inp "d") "errorstate"]
    [(string=? inp "e") "errorstate"]
    [(string=? inp "f") "errorstate"]))

;; after-state1 : MachineInput -> State
;; GIVEN: an input
;; WHERE: so far we've seen (a | b)*c(a|b)*
;; RETURNS: a state that describes what we've seen after the given input
;; EXAMPLES: see next-state
;; STRATEGY: Use template for MachineInput on inp
(define (after-state1 inp)
  (cond
    [(string=? inp "a") "state1"]
    [(string=? inp "b") "state1"]
    [(string=? inp "c") "errorstate"]
    [(string=? inp "d") "state2"]
    [(string=? inp "e") "errorstate"]
    [(string=? inp "f") "errorstate"]))


;; similarly:
;; after-state2 : MachineInput -> State
;; GIVEN: an input
;; WHERE: what we've seen so far matches (a | b)*c(a|b)*d(e|f)*
;; RETURNS: a state that describes what we've seen after the given
;; input
;; EXAMPLES: see next-state
;; STRATEGY: Use template for MachineInput on inp
(define (after-state2 inp)
  (cond
    [(string=? inp "a") "errorstate"]
    [(string=? inp "b") "errorstate"]
    [(string=? inp "c") "errorstate"]
    [(string=? inp "d") "errorstate"]
    [(string=? inp "e") "state2"]
    [(string=? inp "f") "state2"]))



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; TESTS

;; We have 4 states and 6 inputs, so we have 24 equivalence
;; classes to check

;; we'll eliminate boilerplate by writing a help function and defining some constants.

 ;; State Input State -> Boolean
(define (fsm-check s inp out)
  (check-equal? (next-state s inp) out
                (format "(next-state ~a ~a) should be ~a" s inp out)))

;; I get tired of writing those quoted strings.
;; I should have made these constants from the start.
(define st0 "state0")
(define st1 "state1")
(define st2 "state2")
(define err "errorstate")

(define a "a")
(define b "b")
(define c "c")
(define d "d")
(define ee "e")  ; e is already defined as 2.718...
(define f "f")

(begin-for-test

  (fsm-check st0 a st0)
  (fsm-check st0 b st0)
  (fsm-check st0 c st1)
  (fsm-check st0 d err)
  (fsm-check st0 ee err)
  (fsm-check st0 f err)

  (fsm-check st1 a st1)
  (fsm-check st1 b st1)
  (fsm-check st1 c err)
  (fsm-check st1 d st2)
  (fsm-check st1 ee err)
  (fsm-check st1 f err)

  (fsm-check st2 a err)
  (fsm-check st2 b err)
  (fsm-check st2 c err)
  (fsm-check st2 d err)
  (fsm-check st2 ee st2)
  (fsm-check st2 f st2)

  (fsm-check err a err)
  (fsm-check err b err)
  (fsm-check err c err)
  (fsm-check err d err)
  (fsm-check err ee err)
  (fsm-check err f err))

;; Note 1:
;; When I ran this the first time, I got 5 failures. 3 were errors in
;; the tests and 2 were errors in the code.

;; Moral: sometimes it's your tests that are wrong, but sometimes it's
;; really your code.  Don't just change the tests to match what your
;; code produces.

;; Note 2:
;; Last Fall, when I ran this in class, I realized that I had gotten the
;; interpretation in the data definition wrong.  So my tests had all
;; passed, but they were tests for the wrong problem!  Morals:

;; Moral A: the interpretation tells you what you need to know!

;; Moral B: Be sure to get the data definitions and templates right.
;; If either of them is wrong, you will be going down the wrong path
;; for a long, long time. 
