;; The first three lines of this file were inserted by DrRacket. They record metadata
;; about the language level of this file in a form that our tools can easily process.
#reader(lib "htdp-beginner-reader.ss" "lang")((modname robot-directions-better) (read-case-sensitive #t) (teachpacks ()) (htdp-settings #(#t constructor repeating-decimal #f #t none #f () #f)))
;; bits of the robot problem

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; DATA DEFINITIONS
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; A Direction is one of:
;; -- "north"
;; -- "east"
;; -- "south"
;; -- "west"

;; INTERP: self-evident

;; TEMPLATE
;; dir-fn : Direction -> ??
;; (define (dir-fn d)
;;   (cond
;;     [(string=? d "north") ...]
;;     [(string=? d "east") ...]
;;     [(string=? d "south") ...]
;;     [(string=? d "west") ...]))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define-struct robot (x y dir))

;; A Robot is a (make-robot Real Real Direction)
;; INTERP:
;; x and y are the position of the center of the robot
;; dir is the direction the robot is facing

;; TEMPLATE
;; robot-fn : Robot -> ??
;; (define (robot-fn r)
;;   (... (robot-x r) (robot-y r) (robot-dir r)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; Let's look at a couple of  versions of robot-turned-left

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; robot-turned-left (version 0, not acceptable)
;; robot-turned-left : Robot -> Robot
;; GIVEN: a Robot
;; RETURNS: a Robot like the given one, but turned 90 degrees to the
;; left
;; STRATEGY: Use template for Robot on r
(define (robot-turned-left r)
  (cond
    [(robot-north? r) (make-robot (robot-x r) (robot-y r) "west")]
    [(robot-east? r)  (make-robot (robot-x r) (robot-y r) "north")]
    [(robot-south? r) (make-robot (robot-x r) (robot-y r) "east")]
    [(robot-west? r)  (make-robot (robot-x r) (robot-y r) "south")]))

;; Yikes!  That's a great big honking pile of code! Can't we do better?

;; What this code is trying to do is to do a decomposition both on the
;; robot and on its direction.

;; So you need to do this in two steps.
     
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; robot-turned-left (version 1)

;; To do this, we need to analyze a robot AND its direction (2 uses
;; of templates

;; robot-turned-left : Robot -> Robot
;; GIVEN: a Robot
;; RETURNS: a Robot like the given one, but turned 90 degrees to the
;; left
;; STRATEGY: Use template for Robot on r
(define (robot-turned-left r)
  (robot-turned-left-helper (robot-x r) (robot-y r) (robot-dir r)))

;; robot-turned-left-helper : Real Real Direction -> Robot
;; GIVEN: the coordinates and direction of some robot
;; RETURNS: a Robot like the original, but turned 90 degrees to the
;; left.
;; STRATEGY: Use template for Direction on dir
(define (robot-turned-left-helper x y d)
  (cond
    [(string=? dir "north") (make-robot x y "west")]
    [(string=? dir "east")  (make-robot x y "north")]
    [(string=? dir "south") (make-robot x y "east")]
    [(string=? dir "west")  (make-robot x y "south")]))

;; The purpose statement, like:

;; GIVEN: the coordinates and direction of _some robot_

;; tells you what you've figured out before you got to this help
;; function.

;; If you were doing something complicated here, you can and should
;; call another layer of helpers.  But helpers and helpers for
;; helpers, lead to a different kind of mess:  spaghetti code!

;; The fact that I couldn't think of a better name than
;; whatever-helper is a sign that something is starting to smell.


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; robot-turned-left (version 2)

;;; Hmm, all these functions end up doing (make-robot x y
;;; <something>), and the <something> depends only on the direction of
;;; the robot.

;;; With a little cleverness, we can take advantage of
;;; this fact.

;; strategy: Use template for Robot on r
(define (robot-turned-left r)
  (make-robot
    (robot-x r)
    (robot-y r)
    (direction-left (robot-dir r))))



;; direction-left : Direction -> Direction
;; GIVEN: A Direction
;; RETURNS: the Direction 90 degrees to the left of the given one
;; STRATEGY: Use template for Direction on d
(define (direction-left d)
  (cond
    [(string=? d "north") "west"]
    [(string=? d "east")  "north"]
    [(string=? d "south") "east"]
    [(string=? d "west")  "south"]))

;; We'll have a lot more to say about this when we talk about
;; Generalization in Week 5.

;; if you _know_ that robot-fn is always going to return a new robot,
;; you can always write it this way:

;; STRATEGY: Use template for Robot on r
(define (robot-fn r)
  (make-robot
   (robot-fn-new-x-val (robot-x r) .....)
   (robot-fn-new-y-val ..fields of r...)
   (robot-fn-new-dir   ..fields of r ...)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

